import { TemplateResult } from './TemplateResult';
import * as ntriples from './syntax/ntriples';
export class NQuadsTemplateResult extends TemplateResult {
    // eslint-disable-next-line no-useless-constructor
    constructor(strings, values, turtle) {
        super(strings, values, turtle);
    }
    _evaluateDataset(dataset, options) {
        const [first, ...rest] = dataset;
        return rest.reduce((result, quad) => {
            const nextQuad = this._evaluateQuad(quad, options);
            return {
                value: `${result.value}\n${nextQuad.value}`,
                prefixes: result.prefixes,
            };
        }, this._evaluateQuad(first, options));
    }
    _evaluateQuad(quad, options) {
        const subject = this._evaluateTerm(quad.subject, options);
        const predicate = this._evaluateTerm(quad.predicate, options);
        const object = this._evaluateTerm(quad.object, options);
        const graph = this._evaluateTerm(quad.graph, options);
        return {
            value: `${subject.value} ${predicate.value} ${object.value} ${graph.value} .`,
            prefixes: [],
        };
    }
    _evaluateLiteral(term) {
        return {
            value: ntriples.literal(term),
            prefixes: [],
        };
    }
    _evaluateNamedNode(term) {
        return {
            value: ntriples.namedNode(term),
            prefixes: [],
        };
    }
    _evaluateBlankNode(term) {
        return {
            value: ntriples.blankNode(term),
            prefixes: [],
        };
    }
    get __defaultOptions() {
        return {
            sortGraphs: false,
        };
    }
    _getFinalString(result) {
        return result;
    }
}
export const nquads = (strings, ...values) => new NQuadsTemplateResult(strings, values, nquads);
