import { prefixes as knownPrefixes } from '@zazuko/rdf-vocabularies';
import { TemplateResult } from './TemplateResult';
import * as turtleSyntax from './syntax/turtle';
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in knownPrefixes)
        .map(prefix => `PREFIX ${prefix}: <${knownPrefixes[prefix]}>`);
}
export class SparqlTemplateResult extends TemplateResult {
    // eslint-disable-next-line no-useless-constructor
    constructor(strings, values, turtle) {
        super(strings, values, turtle);
    }
    get __defaultOptions() {
        return {
            prologue: true,
        };
    }
    _evaluateLiteral(term, options) {
        return turtleSyntax.literal(term, options.base);
    }
    _evaluateNamedNode(term, options) {
        return turtleSyntax.namedNode(term, options.base);
    }
    _evaluateBlankNode(term) {
        return {
            value: turtleSyntax.blankNode(term),
            prefixes: [],
        };
    }
    _evaluateVariable(term) {
        return {
            value: `?${term.value}`,
            prefixes: [],
        };
    }
    _getFinalString(result, prefixes, options) {
        const prologue = options.prologue || typeof options.prologue === 'undefined';
        let prologueLines = [];
        if (prologue) {
            prologueLines = prefixDeclarations(prefixes);
            if (options.base) {
                prologueLines = [`BASE <${options.base}>`, ...prologueLines];
            }
            if (prologueLines.length > 0) {
                prologueLines.push('\n');
            }
        }
        return `${prologueLines.join('\n')}${result}`;
    }
}
export const sparql = (strings, ...values) => new SparqlTemplateResult(strings, values, sparql);
