import { defaultGraph } from '@rdfjs/data-model';
import { prefixes as knownPrefixes } from '@zazuko/rdf-vocabularies';
import { TemplateResult } from './TemplateResult';
import * as syntax from './syntax/turtle';
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in knownPrefixes)
        .map(prefix => `@prefix ${prefix}: <${knownPrefixes[prefix]}> .`);
}
export class TurtleTemplateResult extends TemplateResult {
    // eslint-disable-next-line no-useless-constructor
    constructor(strings, values, turtle) {
        super(strings, values, turtle);
    }
    get __defaultOptions() {
        return {
            directives: true,
            graph: defaultGraph(),
        };
    }
    _evaluateBlankNode(term) {
        return {
            value: syntax.blankNode(term),
            prefixes: [],
        };
    }
    _evaluateLiteral(term) {
        return syntax.literal(term);
    }
    _evaluateNamedNode(term) {
        return syntax.namedNode(term);
    }
    _evaluateDataset(dataset, options) {
        const graphQuads = [...dataset.match(null, null, null, options.graph)];
        return graphQuads.reduce((result, quad) => {
            const quadResult = this._evaluateQuad(quad, options);
            return {
                value: result.value + '\n' + quadResult.value,
                prefixes: [...result.prefixes, ...quadResult.prefixes],
            };
        }, { value: '', prefixes: [] });
    }
    _evaluateQuad(quad, options) {
        if (!options.graph.equals(quad.graph)) {
            return {
                value: '',
                prefixes: [],
            };
        }
        const subject = this._evaluateTerm(quad.subject, options);
        const predicate = this._evaluateTerm(quad.predicate, options);
        const object = this._evaluateTerm(quad.object, options);
        return {
            value: `${subject.value} ${predicate.value} ${object.value} .`,
            prefixes: [
                ...subject.prefixes,
                ...predicate.prefixes,
                ...object.prefixes,
            ],
        };
    }
    _getFinalString(result, prefixes, options) {
        const prologue = options.directives || typeof options.directives === 'undefined';
        let prologueLines = [];
        if (prologue) {
            prologueLines = prefixDeclarations(prefixes);
            if (prologueLines.length > 0) {
                prologueLines.push('\n');
            }
        }
        return `${prologueLines.join('\n')}${result}`;
    }
}
export const turtle = (strings, ...values) => new TurtleTemplateResult(strings, values, turtle);
