"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class TemplateResult {
    constructor(strings, values, tag) {
        this.strings = strings;
        this.values = values;
        this._tag = tag;
    }
    _evaluateLiteral(term, options) {
        throw new Error(`${this._tag.name} cannot interpolate literals`);
    }
    _evaluateNamedNode(term, options) {
        throw new Error(`${this._tag.name} cannot interpolate named nodes`);
    }
    _evaluateVariable(term) {
        throw new Error(`${this._tag.name} cannot interpolate variables`);
    }
    _evaluateBlankNode(term) {
        throw new Error(`${this._tag.name} cannot interpolate blank nodes`);
    }
    _evaluateQuad(quad, options) {
        throw new Error(`${this._tag.name} cannot interpolate quads`);
    }
    _evaluateDataset(dataset, options) {
        throw new Error(`${this._tag.name} cannot interpolate datasets`);
    }
    toString(options) {
        let actualOptions = this.__defaultOptions;
        if (options) {
            actualOptions = Object.assign(Object.assign({}, actualOptions), options);
        }
        const { value, prefixes } = this._toPartialString(actualOptions);
        return this._getFinalString(value, prefixes, actualOptions);
    }
    _toPartialString(options) {
        const prefixes = new Set();
        const l = this.strings.length - 1;
        let result = '';
        for (let i = 0; i < l; i++) {
            let partialResult = {
                value: '',
                prefixes: [],
            };
            result += this.strings[i];
            const value = this.values[i];
            if (!value)
                continue;
            if (typeof value === 'string') {
                result += value;
            }
            else if (value instanceof TemplateResult) {
                partialResult = value._toPartialString(options);
            }
            else if ('subject' in value) {
                partialResult = this._evaluateQuad(value, options);
            }
            else if ('match' in value) {
                partialResult = this._evaluateDataset(value, options);
            }
            else {
                partialResult = this._evaluateTerm(value, options);
            }
            result += partialResult.value;
            [...partialResult.prefixes].forEach(prefix => prefixes.add(prefix));
        }
        result += this.strings[l];
        return {
            value: result,
            prefixes,
        };
    }
    _evaluateTerm(value, options) {
        switch (value.termType) {
            case 'Literal':
                return this._evaluateLiteral(value, options);
            case 'NamedNode':
                return this._evaluateNamedNode(value, options);
            case 'BlankNode':
                return this._evaluateBlankNode(value);
            case 'Variable':
                return this._evaluateVariable(value);
        }
        return {
            value: '',
            prefixes: [],
        };
    }
}
exports.TemplateResult = TemplateResult;
