"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const rdf_vocabularies_1 = require("@zazuko/rdf-vocabularies");
const TemplateResult_1 = require("./TemplateResult");
const turtleSyntax = __importStar(require("./syntax/turtle"));
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in rdf_vocabularies_1.prefixes)
        .map(prefix => `PREFIX ${prefix}: <${rdf_vocabularies_1.prefixes[prefix]}>`);
}
class SparqlTemplateResult extends TemplateResult_1.TemplateResult {
    // eslint-disable-next-line no-useless-constructor
    constructor(strings, values, turtle) {
        super(strings, values, turtle);
    }
    get __defaultOptions() {
        return {
            prologue: true,
        };
    }
    _evaluateLiteral(term, options) {
        return turtleSyntax.literal(term, options.base);
    }
    _evaluateNamedNode(term, options) {
        return turtleSyntax.namedNode(term, options.base);
    }
    _evaluateBlankNode(term) {
        return {
            value: turtleSyntax.blankNode(term),
            prefixes: [],
        };
    }
    _evaluateVariable(term) {
        return {
            value: `?${term.value}`,
            prefixes: [],
        };
    }
    _getFinalString(result, prefixes, options) {
        const prologue = options.prologue || typeof options.prologue === 'undefined';
        let prologueLines = [];
        if (prologue) {
            prologueLines = prefixDeclarations(prefixes);
            if (options.base) {
                prologueLines = [`BASE <${options.base}>`, ...prologueLines];
            }
            if (prologueLines.length > 0) {
                prologueLines.push('\n');
            }
        }
        return `${prologueLines.join('\n')}${result}`;
    }
}
exports.SparqlTemplateResult = SparqlTemplateResult;
exports.sparql = (strings, ...values) => new SparqlTemplateResult(strings, values, exports.sparql);
