"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const data_model_1 = require("@rdfjs/data-model");
const rdf_vocabularies_1 = require("@zazuko/rdf-vocabularies");
const TemplateResult_1 = require("./TemplateResult");
const syntax = __importStar(require("./syntax/turtle"));
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in rdf_vocabularies_1.prefixes)
        .map(prefix => `@prefix ${prefix}: <${rdf_vocabularies_1.prefixes[prefix]}> .`);
}
class TurtleTemplateResult extends TemplateResult_1.TemplateResult {
    // eslint-disable-next-line no-useless-constructor
    constructor(strings, values, turtle) {
        super(strings, values, turtle);
    }
    get __defaultOptions() {
        return {
            directives: true,
            graph: data_model_1.defaultGraph(),
        };
    }
    _evaluateBlankNode(term) {
        return {
            value: syntax.blankNode(term),
            prefixes: [],
        };
    }
    _evaluateLiteral(term) {
        return syntax.literal(term);
    }
    _evaluateNamedNode(term) {
        return syntax.namedNode(term);
    }
    _evaluateDataset(dataset, options) {
        const graphQuads = [...dataset.match(null, null, null, options.graph)];
        return graphQuads.reduce((result, quad) => {
            const quadResult = this._evaluateQuad(quad, options);
            return {
                value: result.value + '\n' + quadResult.value,
                prefixes: [...result.prefixes, ...quadResult.prefixes],
            };
        }, { value: '', prefixes: [] });
    }
    _evaluateQuad(quad, options) {
        if (!options.graph.equals(quad.graph)) {
            return {
                value: '',
                prefixes: [],
            };
        }
        const subject = this._evaluateTerm(quad.subject, options);
        const predicate = this._evaluateTerm(quad.predicate, options);
        const object = this._evaluateTerm(quad.object, options);
        return {
            value: `${subject.value} ${predicate.value} ${object.value} .`,
            prefixes: [
                ...subject.prefixes,
                ...predicate.prefixes,
                ...object.prefixes,
            ],
        };
    }
    _getFinalString(result, prefixes, options) {
        const prologue = options.directives || typeof options.directives === 'undefined';
        let prologueLines = [];
        if (prologue) {
            prologueLines = prefixDeclarations(prefixes);
            if (prologueLines.length > 0) {
                prologueLines.push('\n');
            }
        }
        return `${prologueLines.join('\n')}${result}`;
    }
}
exports.TurtleTemplateResult = TurtleTemplateResult;
exports.turtle = (strings, ...values) => new TurtleTemplateResult(strings, values, exports.turtle);
